import numpy as np
import matplotlib.pyplot as pl

def plot_stability_region(p,q,N=200,bounds=[-10,1,-5,5],
                color='r',filled=True,scaled=False,plotroots=True,
                alpha=1.,scalefac=None):
    r""" 
        The region of absolute stability
        of a rational function; i.e. the set

        `\{ z \in C : |\phi (z)|\le 1 \}`

        **Input**: (all optional)
            - N       -- Number of gridpoints to use in each direction
            - bounds  -- limits of plotting region
            - color   -- color to use for this plot
            - filled  -- if true, stability region is filled in (solid); otherwise it is outlined
    """
    m=len(p)
    x=np.linspace(bounds[0],bounds[1],N)
    y=np.linspace(bounds[2],bounds[3],N)
    X=np.tile(x,(N,1))
    Y=np.tile(y[:,np.newaxis],(1,N))
    Z=X+Y*1j
    if scaled: 
        if scalefac==None: scalefac=m
    else: scalefac=1.
    R=np.abs(p(Z*scalefac)/q(Z*scalefac))
    #pl.clf()
    if filled:
        pl.contourf(X,Y,R,[0,1],colors=color,alpha=alpha)
    else:
        pl.contour(X,Y,R,[0,1],colors=color,alpha=alpha)
    pl.title('Absolute Stability Region')
    pl.hold(True)
    if plotroots: pl.plot(np.real(p.r),np.imag(p.r),'ok')
    if len(q)>1: pl.plot(np.real(q.r),np.imag(q.r),'xk')
    pl.plot([0,0],[bounds[2],bounds[3]],'--k',linewidth=2)
    pl.plot([bounds[0],bounds[1]],[0,0],'--k',linewidth=2)
    pl.axis('Image')
    pl.hold(False)
    pl.draw()

def plot_order_star(p,q,N=200,bounds=[-5,5,-5,5],
                color=('w','b'),filled=True,subplot=None):
    r""" Plot the order star of a rational function
        i.e. the set
        
        $$ \{ z \in C : |R(z)/exp(z)|\le 1 \} $$

        where $R(z)=p(z)/q(z)$ is the stability function of the method.

        **Input**: (all optional)
            - N       -- Number of gridpoints to use in each direction
            - bounds  -- limits of plotting region
            - color   -- color to use for this plot
            - filled  -- if true, order star is filled in (solid); otherwise it is outlined
    """
    x=np.linspace(bounds[0],bounds[1],N)
    y=np.linspace(bounds[2],bounds[3],N)
    X=np.tile(x,(N,1))
    Y=np.tile(y[:,np.newaxis],(1,N))
    Z=X+Y*1j
    R=np.abs(p(Z)/q(Z)/np.exp(Z))
    if subplot is not None:
        pl.subplot(subplot[0],subplot[1],subplot[2])
    else:
        pl.clf()
    pl.contourf(X,Y,R,[0,1,1.e299],colors=color)
    pl.hold(True)
    pl.plot([0,0],[bounds[2],bounds[3]],'--k')
    pl.plot([bounds[0],bounds[1]],[0,0],'--k')
    pl.axis('Image')
    pl.hold(False)
    pl.draw()


def pade_exp(k,j):
    """
    Return the Pade approximation to the exponential function
    with numerator of degree k and denominator of degree j.
    """
    Pcoeffs=[1]
    Qcoeffs=[1]
    for n in range(1,k+1):
        newcoeff=Pcoeffs[0]*(k-n+1.)/(j+k-n+1.)/n
        Pcoeffs=[newcoeff]+Pcoeffs
    P=pl.poly1d(Pcoeffs)
    for n in range(1,j+1):
        newcoeff=-1.*Qcoeffs[0]*(j-n+1.)/(j+k-n+1.)/n
        Qcoeffs=[newcoeff]+Qcoeffs
    Q=pl.poly1d(Qcoeffs)
    return P,Q

def taylor(p):
    r"""
        Return the Taylor polynomial of the exponential, up to order p.
    """
    from scipy import factorial

    coeffs=1./factorial(np.arange(p+1))

    return np.poly1d(coeffs[::-1])

def plot_stability_region(p,q,N=200,bounds=[-10,1,-5,5],
                color='r',filled=True,scaled=False,plotroots=True,
                alpha=1.,scalefac=None):
    r""" 
        The region of absolute stability
        of a Runge-Kutta method, is the set

        `\{ z \in C : |\phi (z)|\le 1 \}`

        where $\phi(z)$ is the stability function of the method.

        **Input**: (all optional)
            - N       -- Number of gridpoints to use in each direction
            - bounds  -- limits of plotting region
            - color   -- color to use for this plot
            - filled  -- if true, stability region is filled in (solid); otherwise it is outlined
    """
    m=len(p)
    x=np.linspace(bounds[0],bounds[1],N)
    y=np.linspace(bounds[2],bounds[3],N)
    X=np.tile(x,(N,1))
    Y=np.tile(y[:,np.newaxis],(1,N))
    Z=X+Y*1j
    if scaled: 
        if scalefac==None: scalefac=m
    else: scalefac=1.
    R=np.abs(p(Z*scalefac)/q(Z*scalefac))
    pl.clf()
    if filled:
        pl.contourf(X,Y,R,[0,1],colors=color,alpha=alpha)
    else:
        pl.contour(X,Y,R,[0,1],colors=color,alpha=alpha)
    pl.title('Absolute Stability Region')
    pl.hold(True)
    if plotroots: pl.plot(np.real(p.r),np.imag(p.r),'ok')
    if len(q)>1: pl.plot(np.real(q.r),np.imag(q.r),'xk')
    pl.plot([0,0],[bounds[2],bounds[3]],'--k',linewidth=2)
    pl.plot([bounds[0],bounds[1]],[0,0],'--k',linewidth=2)
    pl.axis('Image')
    pl.hold(False)
    pl.draw()





import networkx as nx
import numpy as np

def longest_paths(G):
    """Return a dict of nodes with values corresponding to longest paths from node"""
    lp = {}
    lp_old = lp.copy()
    for node in G.nodes_iter():
        lp[node] = 0

    while lp != lp_old:
        lp_old = lp.copy()
        for edge in G.edges_iter():
            lp[edge[1]] = max(lp[edge[1]],lp[edge[0]]+1)

    return lp

def rk_dependency_graph(rkm):
    """Dependency graph of Runge-Kutta method stages"""
    s = len(rkm)
    K = np.zeros((s+1,s+1))
    K[:,:-1] = np.vstack((rkm.A,rkm.b))
    G = nx.from_numpy_matrix(np.abs(K.T)>0,nx.DiGraph())
    return G

def remove_extra_edges(G):
    lp = longest_paths(G)
    to_remove = []
    for edge in G.edges_iter():
        if lp[edge[0]]<(lp[edge[1]]-1):
            to_remove.append(edge)
    for edge in to_remove:
        G.remove_edge(edge[0],edge[1])

def plot_dependency_graph(rkm,remove_edges=False):
    G = rk_dependency_graph(rkm)
    lp = longest_paths(G)

    hpos = [0]*(max(lp)+1)
    pos = {}
    for node in G.nodes_iter():
        hpos[lp[node]]+=1
        pos[node] = np.array([hpos[lp[node]],lp[node]])

    if remove_edges:
       remove_extra_edges(G)

    nx.draw_networkx(G,pos)



"""
Unit tests for NodePy.
This needs to be updated.
"""
import linear_multistep_method as lmm
import runge_kutta_method as rk
import unittest as ut

class LinearMultistepTestCase(ut.TestCase):
    pass

class LMMOrderTest(LinearMultistepTestCase):
    def runTest(self):
        for k in range(1,7):
            ab=lmm.Adams_Bashforth(k)
            self.assertEqual(ab.order(),k)
            am=lmm.Adams_Moulton(k)
            self.assertEqual(am.order(),k+1)
            bdf=lmm.backward_difference_formula(k)
            self.assertEqual(bdf.order(),k)

class LMMSSPCoeffTest(LinearMultistepTestCase):
    def runTest(self):
        for k in range(2,100):
            ssp2=lmm.elmm_ssp2(k)
            self.assertAlmostEqual(ssp2.ssp_coefficient(),(k-2.)/(k-1.),10)

class RungeKuttaTestCase(ut.TestCase):
    def setUp(self):
        self.RKs=rk.loadRKM()

class RKOrderTest(RungeKuttaTestCase):
    knownValues = ( ('FE',1),
            ('SSP22',2),
            ('SSP33',3),
            ('Mid22',2),
            ('RK44',4),
            ('SSP104',4),
            ('GL2',4),
            ('GL3',6),
            ('BuRK65',5) )

    def runTest(self):
        for method, order in self.knownValues:
            self.assertEqual(self.RKs[method].order(),order)

class RKStageOrderTest(RungeKuttaTestCase):
    knownValues = ( ('FE',1),
            ('SSP22',1),
            ('SSP33',1),
            ('Mid22',1),
            ('RK44',1),
            ('SSP104',1),
            ('GL2',2),
            ('GL3',3),
            ('BuRK65',1) )

    def runTest(self):
        for method, stageorder in self.knownValues:
            self.assertEqual(self.RKs[method].stage_order(),stageorder)

class RKAmradTest(RungeKuttaTestCase):
    knownValues = ( ('FE',1),
            ('SSP22',1),
            ('SSP33',1),
            ('Mid22',0),
            ('RK44',0) ,
            ('SSP104',6),
            ('BuRK65',0) )

    def runTest(self):
        for method, SSPCoefficient in self.knownValues:
            self.assertAlmostEqual(self.RKs[method].absolute_monotonicity_radius(),SSPCoefficient,9)

class linAmradTest(RungeKuttaTestCase):
    knownValues = ( ('FE',1),
            ('SSP22',1),
            ('SSP33',1),
            ('Mid22',1),
            ('RK44',1) ,
            ('SSP104',6),
            ('BuRK65',16/9.) )

    def runTest(self):
        for method, R in self.knownValues:
            self.assertAlmostEqual(self.RKs[method].linear_absolute_monotonicity_radius(),R,2)

if __name__== "__main__":
    ut.main()



"""
NodePy (Numerical ODE solvers in Python) is...
"""

__version__="0.4"

import sys
mypath= '/Users/ketch/Research/Projects/nodepy'
if mypath not in sys.path: sys.path.append(mypath)

import runge_kutta_method as rk
import linear_multistep_method as lm
import rooted_trees as rt
import ivp
import convergence as conv
import low_storage_rk as lsrk



def bisect(rlo, rhi, acc, tol, fun, params=None):
    """ 
        Performs a bisection search.

        **Input**:
            - fun -- a function such that fun(r)==True iff x_0>r, where x_0 is the value to be found.
    """
    while rhi-rlo>acc:
        r=0.5*(rhi+rlo)
        if params: isvalid=fun(r,tol,params)
        else: isvalid=fun(r,tol)
        if isvalid:
            rlo=r
        else:
            rhi=r
    return rlo

def permutations(str):
    if len(str) <=1:
        yield str
    else:
        for perm in permutations(str[1:]):
            for i in range(len(perm)+1):
                yield perm[:i] + str[0:1] + perm[i:]

def shortstring(x):
    import numpy as np
    import sympy.core.numbers
    if x.__class__ is np.float64 or x.__class__ is sympy.Float:
        return '%6.3f' % x
    else: 
        return ' '+str(x)


def find_plot_bounds(f,guess,N=101,zmax=1000):
    r"""Find reasonable area to plot for stability regions.
    
    Tries to find an area that contains the entire stability region
    but isn't too big.  Makes lots of assumptions.  Obviously can't
    work for unbounded stability regions.

    f should return True if f(z) is in the stability region.

    N should be odd in order to catch very small stability regions.
    """
    import numpy as np

    bounds = guess
    old_bounds = []

    while bounds != old_bounds:
        old_bounds = bounds
        y=np.linspace(bounds[2],bounds[3],N)

        #Check boundaries
        bounds = list(bounds)
        close = False
        while abs(bounds[0])<zmax:
            x=np.linspace(bounds[0],bounds[1],N)
            Z=x[0]+y*1j

            left = f(Z)
            if np.any(left):
                bounds[0] = 1.5*bounds[0]
                close = True
            else:
                if close == True:
                    break
                bounds[0] = bounds[0]/1.5
                if bounds[0] > -1.e-15:
                    raise Exception('No stable region found; is this method zero-stable?')

        bounds[1] = -0.1*bounds[0]
        x=np.linspace(bounds[0],bounds[1],N) + 0.*1j

        close = False
        while abs(bounds[2])<zmax:
            y=np.linspace(bounds[2],bounds[3],N)
            Z=x + y[0]*1j

            bottom = f(Z)
            if np.any(bottom):
                bounds[2] = 1.5*bounds[2]
                close = True
            else:
                if close == True:
                    break
                bounds[2] = bounds[2]/1.5
                if bounds[2] > -1.e-15:
                    raise Exception('No stable region found; is this method zero-stable?')

        bounds[3] = -bounds[2]

    return bounds

 



#ISSUES:
#    - Need a way to pass things like dx
#    - Need to call boundary conditions separately since they aren't always
#        written as ODEs

#Should have a special class for linear semidiscretizations?
"""
In NodePy a semi-discretization is a family of IVPs parameterized by grid size.
For now, only semi-discretizations of one-dimensional PDEs are supported.
"""

import numpy as np
from ivp import IVP

class LinearSemiDiscretization(IVP):
    """
        Class for linear semi-discretizations of PDEs.
        Inherits from IVP, but possesses a grid and
        is parameterized by grid size.

        Any instance should provide:

        - L: the matrix representation of the right-hand-side
        - N, xmin, xmax (describing the grid)
    """

def load_semidisc(sdName,N=100,xMin=0.,xMax=1.,order=1):
    sd=LinearSemiDiscretization()

    #Set up grid
    ############
    # Number of cells
    sd.N=N

    # Grid spacing
    dx=(xMax-xMin)/N;                               

    # Cell centers position
    sd.xCenter = np.zeros(N)
    sd.xCenter[0] = xMin + dx/2.0
    for i in range(1,N):
        sd.xCenter[i] = sd.xCenter[0]+dx*i

    # The spatial discretization sets also the initial condition
    # because the number of DOF depends on the scheme
    if sdName=='upwind advection':
        sd.L  = upwind_advection_matrix(N,dx)
        sd.u0 = np.sin(2*np.pi*sd.xCenter)
    elif sdName == 'spectral difference advection':
        sd.L,sd.xSol,sd.u0 = spectral_difference_matrix(N,sd.xCenter,dx,order)
    else: print 'unrecognized sdname'
    
    # Define RHS
    # In this case is simply L*u
    sd.rhs=lambda t,u : np.dot(sd.L,u)
    
    # Set final time
    sd.T = 5.9


    # Construct exact solution by using a large number of cells. 
    # Probably this is not the best place to construct it because if one want
    # to change the the initial solution he must also change the intial condition
    # in the spatial discretization.
    #############################################################################
    nbrCellsExact = 2000
    sd.xExact = np.linspace(xMin,xMax,nbrCellsExact)   # Position of the interfaces
    sd.uExactInit = np.zeros(nbrCellsExact)
    sd.uExactInit = np.sin(2*np.pi*sd.xExact)

    # Compute the real final position of the last point. 
    # One point is enough because it is a rigid translation in the x direction
    xExactTmp = sd.xExact+sd.T
    xLastPnt = xExactTmp[nbrCellsExact-1]
  
    from math import modf

    b,a = modf(xLastPnt)

    
    # 1st approach: valid for classical periodic initial condition, e.g. sin(), cos(), etc.
    sd.uExact = np.zeros((nbrCellsExact))
    sd.uExact = np.sin(2*np.pi*(sd.xExact-b))
    
    # 2nd approach: general
    ##############
    #if b != 0.:
    #    # Copy part of the solution on the left
    #    for i in range(0,nbrCellsExact):
    #        if  xExactTmp[i] > a:
    #            index = i
    #            break
    #    # New exact solution with periodic BC
    #    sd.uExact[0] = sd.uExactInit[index-1]
    #    sd.uExact[1:nbrCellsExact-index+1] = sd.uExactInit[index:nbrCellsExact]
    #    sd.uExact[nbrCellsExact-index+1:nbrCellsExact+1] = sd.uExactInit[0:index]
    #        
    #    sd.xExact = np.linspace(xMin,xMax,nbrCellsExact+1)
    #else:
    #    sd.uExact = sd.uExact
    #    sd.xExact = np.linspace(xMin,xMax,nbrCellsExact)

    return sd


def upwind_advection_matrix(N,dx):
    #from scipy.sparse import spdiags
    e=np.ones(N)
    #L=spdiags([-e,e,e],[0,-1,N-1],N,N)/dx
    L=(np.diag(-e)+np.diag(e[1:],-1))
    L[0,-1]=1.
    L=L/dx
    return L


def centered_advection_matrix(N,dx=0):
    "3-point centered difference approximation of first derivative."
    if dx==0: dx=1./N
    e=np.ones(N)
    L=(np.diag(-0.5*e[1:],1)+np.diag(0.5*e[1:],-1))
    L[0,-1]=0.5
    L[-1,0]=-0.5
    L=L/dx
    return L

def centered_diffusion_matrix(N,dx=0):
    "3-point centered difference approximation of second derivative."
    if dx==0: dx=1./N
    e=np.ones(N)
    L=(np.diag(e[1:],1)-2.*np.diag(e,0)+np.diag(e[1:],-1))
    L[0,-1]=1.
    L[-1,0]=1.
    L=L/dx**2
    return L

def centered_advection_diffusion_matrix(a,b,N,dx=0):
    r"3-point centered difference approximation for `a u_x + b u_{xx}`."
    if dx==0: dx=1./N
    L1 = centered_advection_matrix(N,dx)
    L2 = centered_diffusion_matrix(N,dx)
    L = a*L1 + b*L2
    return L


def spectral_difference_matrix(nbrCells,xCenter,dx,order):
    np.set_printoptions(threshold=np.nan)


    # Set coordinates of flux and solutions points according to the order of the scheme
    if order == 1:
        # 1st order
        fluxPnts = np.array([ -1.0 , 1.0 ])
        solPnts = np.array([ 0 ])
    elif order == 2:
        # 2nd order
        fluxPnts = np.array([ -1.0 , 0.0 , 1.0 ])
        solPnts = np.array([ -1.0 ,       1.0 ])
    elif order == 3:
        # 3rd order
        fluxPnts = np.array([ -1.0 , -0.57 , 0.57 , 1.0 ])
        solPnts = np.array([ -1.0 ,         0.57 , 1.0 ])
    elif order == 4:
        # 4th order
        fluxPnts = np.array([ -1.0 , -0.78 , 0.0 , 0.78 , 1.0 ])
        solPnts = np.array([ -1.0 , -0.78 ,       0.78 , 1.0 ])
    elif order == 5:
        # 5th order
        fluxPnts = np.array([ -1.0 , -0.83 , -0.36 , 0.36 , 0.83 , 1.0 ])
        solPnts = np.array([ -1.0 , -0.83 ,         0.36 , 0.83 , 1.0 ])
    elif order == 6:
        # 6th order
        fluxPnts = np.array([ -1.0 , -0.88 , -0.53 , 0.0 , 0.53 , 0.88 , 1.0 ])
        solPnts = np.array([ -1.0 , -0.88 , -0.53 ,       0.53 , 0.88 , 1.0 ])
    else:
       raise Exception("Error: min order 1, max order 6")


    # Number of solution and flux points
    ####################################
    nbrFluxPnts = fluxPnts.size
    nbrSolPnts = solPnts.size

    # Matrices for the calculation of the solution at the flux points 
    #################################################################
    extrSolToFlux = np.ones((nbrFluxPnts,nbrSolPnts))
    #print extrSolToFlux


    for iFlux in range(0,nbrFluxPnts):
        for iSol in range(0,nbrSolPnts):
            for iCon in range(0,nbrSolPnts):
                if iCon != iSol:
                    extrSolToFlux[iFlux,iSol] = extrSolToFlux[iFlux,iSol]*(fluxPnts[iFlux]-solPnts[iCon])/(solPnts[iSol]-solPnts[iCon])
            
           
  
    # Riemann flux settings
    #######################
    upwindPar = 1.0 # If fully upwind (upwindPar =1). For advection equation upwindPar = 1.0

    faceLeft  = 0.5*(1.0 + upwindPar); # Contribution of the left solution 
    faceRight = 0.5*(1.0 - upwindPar); # Contribution of the right solution


    # Construction of the flux polynomial
    #####################################
    fluxCurrCell = np.zeros((nbrFluxPnts,nbrSolPnts));
    fluxLeftCell = np.zeros((nbrFluxPnts,nbrSolPnts));
    fluxRightCell = np.zeros((nbrFluxPnts,nbrSolPnts));
    
    # Flux at the internal flux points
    for iFlux in range(1,nbrFluxPnts-1):
        for iSol in range(0,nbrSolPnts):
            fluxCurrCell[iFlux,iSol] = extrSolToFlux[iFlux,iSol]  # We assume convective velocity = 1, i.e. the flux function is 
                                                                  # f= a*u



    # Flux at the boundaries flux points
    for iSol in range(0,nbrSolPnts):
        # Left face
        fluxLeftCell[0,iSol] = faceLeft*extrSolToFlux[nbrFluxPnts-1,iSol] 
        fluxCurrCell[0,iSol] = faceRight*extrSolToFlux[0,iSol]
    
        # Right face
        fluxCurrCell[nbrFluxPnts-1,iSol] = faceLeft*extrSolToFlux[nbrFluxPnts-1,iSol]
        fluxRightCell[nbrFluxPnts-1,iSol] = faceRight*extrSolToFlux[0,iSol]


    # Matrix for the calculation of the flux derivative at the solution points
    ##########################################################################
    derivFluxInSolPnts = np.zeros((nbrSolPnts,nbrFluxPnts))

    for iSol in range(0,nbrSolPnts):
        for iFlux in range(0,nbrFluxPnts):
            for iCon in range(0,nbrFluxPnts):
                if iCon != iFlux:
                    contribution = 1.0/(fluxPnts[iFlux]-fluxPnts[iCon])
                    for iFac in range(0,nbrFluxPnts):
                        if (iFac != iCon) & (iFac != iFlux):
                            contribution = contribution*(solPnts[iSol]-fluxPnts[iFac])/(fluxPnts[iFlux]-fluxPnts[iFac])
                    derivFluxInSolPnts[iSol,iFlux] = derivFluxInSolPnts[iSol,iFlux] + contribution
            # Factor 2 comes from Jacobian determinant
            derivFluxInSolPnts[iSol,iFlux] = derivFluxInSolPnts[iSol,iFlux]*2.0/dx



    # Compute the products between the matrix of the derivatives and the solution contributions
    ###########################################################################################
    DMm1 = np.dot(derivFluxInSolPnts,fluxLeftCell)
    DM0  = np.dot(derivFluxInSolPnts,fluxCurrCell)
    DMp1 = np.dot(derivFluxInSolPnts,fluxRightCell)

    # Create block tridiagonal matrix
    #################################  
    dimL = nbrSolPnts*nbrCells
    L = np.zeros((dimL,dimL))

    # Main block diagonal
    for iBlock in range(0,nbrCells):
        L[nbrSolPnts*iBlock:nbrSolPnts*(iBlock+1),nbrSolPnts*iBlock:nbrSolPnts*(iBlock+1)] = DM0[:,:] 
    
    # Lower and upper blocks diagonal
    for iBlock in range(0,nbrCells-1):
        L[nbrSolPnts*(iBlock+1):nbrSolPnts*(iBlock+2),nbrSolPnts*iBlock:nbrSolPnts*(iBlock+1)] = DMm1[:,:]
        L[nbrSolPnts*iBlock:nbrSolPnts*(iBlock+1),nbrSolPnts*(iBlock+1):nbrSolPnts*(iBlock+2)] = DMp1[:,:]


    # Add periodic boundary condition contributions to the operator L
    #################################################################
    # Apply BC to the first cell --> DMm1
    L[0:nbrSolPnts,nbrSolPnts*(nbrCells-1):nbrSolPnts*nbrCells] = DMm1[:,:]    

    # Apply BC to the last cell --> DMp1
    # Actually here we are doing nothing because it is a fully upwind scheme with 1D advection equation 
    L[nbrSolPnts*(nbrCells-1):nbrSolPnts*nbrCells,0:nbrSolPnts] = DMp1[:,:]  


    # Bring spatial discretization on the RHS of the equation
    #########################################################
    L = -1*L
  
    # Construct initial solution
    ############################
    u0 = np.zeros((dimL))

    # Create solution points position
    # Here the solution points at the interface are reated two times because of the
    # spectral difference method
    xSolPnts = np.zeros(dimL)
    for i in range(0,nbrCells):
        for j in range(0,nbrSolPnts):
            xSolPnts[i*nbrSolPnts+j] = xCenter[i] + 1./2.*dx*solPnts[j]

    u0 = np.sin(2*np.pi*xSolPnts)

   
    return L,xSolPnts,u0







"""
Class for two-step Runge-Kutta methods, and various functions related to them.

AUTHOR: David Ketcheson (08-30-2008)

EXAMPLES:

REFERENCES:
    [jackiewicz1995,butcher1997,hairer1997]
"""
from __future__ import division
from general_linear_method import GeneralLinearMethod
import numpy as np
import rooted_trees as tt
from strmanip import *

#=====================================================
class TwoStepRungeKuttaMethod(GeneralLinearMethod):
#=====================================================
    """ General class for Two-step Runge-Kutta Methods 
        The representation
        uses the form and notation of [Jackiewicz1995]_.

        `\\begin{align*}
        y^n_j = & d_j u^{n-1} + (1-d_j)u^n + \\Delta t \\sum_{k=1}^{s}
        (\\hat{a}_{jk} f(y_k^{n-1}) + a_{jk} f(y_k^n)) & (1\\le j \\le s) \\\\
        u^{n+1} = & \\theta u^{n-1} + (1-\\theta)u^n + \\Delta t \\sum_{j=1}^{s}(\\hat{b}_j f(y_j^{n-1}) + b_j f(y_j^n))
        \\end{align*}`
    """
    def __init__(self,d,theta,A,b,Ahat=None,bhat=None,type='Type II',name='Two-step Runge-Kutta Method'):
        r"""
            Initialize a 2-step Runge-Kutta method."""
        self.s = max(np.shape(b))
        self.d,self.theta,self.A,self.b = d,theta,A,b
        self.Ahat,self.bhat=Ahat,bhat
        #if type=='General': self.Ahat,self.bhat=Ahat,bhat
        #elif type=='Type I':
        #    self.Ahat = np.zeros([self.s,self.s])
        #    self.bhat = np.zeros([self.s,1])
        #elif type=='Type II':
        #    self.Ahat = np.zeros([self.s,self.s])
        #    self.Ahat[:,0]=Ahat
        #    self.bhat = np.zeros([self.s,1])
        #    self.bhat[0]=bhat
        #else: raise TwoStepRungeKuttaError('Unrecognized type')
        self.name=name
        self.type=type

    def order(self,tol=1.e-13):
        r""" 
            Return the order of a Two-step Runge-Kutta method.
            Computed by computing the elementary weights corresponding
            to the appropriate rooted trees.
        """
        p=0
        while True:
            z=self.order_conditions(p+1)
            if np.any(abs(z)>tol): return p
            p=p+1

    def order_conditions(self,p):
        r"""
            Evaluate the order conditions corresponding to rooted trees
            of order $p$.

            **Output**:
                - A vector $z$ of residuals (the amount by which each
                  order condition is violated)

            TODO: Implement simple order conditions (a la Albrecht)
                    for Type I & II TSRKs
        """
        from numpy import dot
        d,theta,Ahat,A,bhat,b=self.d,self.theta,self.Ahat,self.A,self.bhat,self.b
        e=np.ones([len(b),1])
        b=b.T; bhat=bhat.T
        c=dot(Ahat+A,e)-d
        code=TSRKOrderConditions(p)
        z=np.zeros(len(code))
        for i in range(len(code)):
            exec('yy='+code[i])
            exec('z[i]='+code[i])
            #print p,z
        return z


    def stability_matrix(self,z):
        r""" 
            Constructs the stability matrix of a two-step Runge-Kutta method.
            Right now just for a specific value of z.
            We ought to use Sage to do it symbolically.

            **Output**:
                M -- stability matrix evaluated at z

            WARNING: This only works for Type I & Type II methods
            right now!!!
        """
        D=np.hstack([1.-self.d,self.d]) 
        thet=np.hstack([1.-self.theta,self.theta])
        A,b=self.A,self.b
        if self.type=='Type II':
            ahat = np.zeros([self.s,1]); ahat[:,0] = self.Ahat[:,0]
            bh = np.zeros([1,1]); bh[0,0]=self.bhat[0]
            A=np.hstack([ahat,self.A])
            A=np.vstack([np.zeros([1,self.s+1]),A])
            b =  np.vstack([bh,self.b])

        M1=np.linalg.solve(np.eye(self.s)-z*self.A,D)
        L1=thet+z*np.dot(self.b.T,M1)
        M=np.vstack([L1,[1.,0.]])
        return M

    def plot_stability_region(self,N=50,bounds=[-10,1,-5,5],
                    color='r',filled=True,scaled=False):
        r""" 
            Plot the region of absolute stability
            of a Two-step Runge-Kutta method, i.e. the set

            `\{ z \in C : M(z) is power bounded \}`

            where $M(z)$ is the stability matrix of the method.

            **Input**: (all optional)
                - N       -- Number of gridpoints to use in each direction
                - bounds  -- limits of plotting region
                - color   -- color to use for this plot
                - filled  -- if true, stability region is filled in (solid); otherwise it is outlined
        """
        import pylab as pl

        x=np.linspace(bounds[0],bounds[1],N)
        y=np.linspace(bounds[2],bounds[3],N)
        X=np.tile(x,(N,1))
        Y=np.tile(y[:,np.newaxis],(1,N))
        Z=X+Y*1j
        R=Z*0
        for i,xx in enumerate(x):
            for j,yy in enumerate(y):
                M=self.stability_matrix(xx+yy*1j)
                R[j,i]=max(abs(np.linalg.eigvals(M)))
                #print xx,yy,R[i,j]
        if filled:
            pl.contourf(X,Y,R,[0,1],colors=color)
        else:
            pl.contour(X,Y,R,[0,1],colors=color)
        pl.title('Absolute Stability Region for '+self.name)
        pl.hold(True)
        pl.plot([0,0],[bounds[2],bounds[3]],'--k',linewidth=2)
        pl.plot([bounds[0],bounds[1]],[0,0],'--k',linewidth=2)
        pl.axis('Image')
        pl.hold(False)

    def absolute_monotonicity_radius(self,acc=1.e-10,rmax=200,
                    tol=3.e-16):
        r""" 
            Returns the radius of absolute monotonicity
            of a TSRK method.
        """
        from utils import bisect
        r=bisect(0,rmax,acc,tol,self.is_absolutely_monotonic)
        return r

    def spijker_form(self):
        r""" Returns arrays $S,T$ such that the TSRK can be written
            $$ w = S x + T f(w),$$
            and such that $\[S \ \ T\]$ has no two rows equal.
        """
        s=self.s
        if self.type=='General':
            z0=np.zeros([s,1])
            z00=np.zeros([1,1])
            T3 = np.hstack([self.Ahat,z0,self.A,z0])
            T4 = np.hstack([self.bhat.T,z00,self.b.T,z00])
            T = np.vstack([np.zeros([s+1,2*s+2]),T3,T4])
            S1 = np.hstack([np.zeros([s+1,1]),np.eye(s+1)])
            S2 = np.hstack([self.d,np.zeros([s,s]),1-self.d])
            S3 = np.hstack([[[self.theta]],np.zeros([1,s]),[[1-self.theta]]])
            S = np.vstack([S1,S2,S3])

        elif self.type=='Type I':
            K = np.vstack([self.A,self.b.T])
            T2 = np.hstack([np.zeros([s+1,1]),K,np.zeros([s+1,1])])
            T =  np.vstack([np.zeros([1,s+1]),T2])
            S1 = np.vstack([np.zeros([1,1]),self.d,np.array([[self.theta]])])
            S2 = np.vstack([np.zeros([1,1]),1-self.d,np.array([[1-self.theta]])])
            S = np.hstack([S1,S2])

        elif self.type=='Type II':
            S0 = np.array([1,0])
            S1 = np.hstack([self.d,1-self.d])
            S2 = np.array([self.theta,1-self.theta])
            S =  np.vstack([S0,S1,S2])
            ahat = np.zeros([s,1])
            ahat[:,0] = self.Ahat[:,0]
            bh = np.zeros([1,1])
            bh[0,0]=self.bhat[0]
            T0 = np.zeros([1,s+2])
            T1 =  np.hstack([ahat,self.A,np.zeros([s,1])])
            T2 =  np.hstack([bh,self.b.T,np.zeros([1,1])])
            T = np.vstack([T0,T1,T2])
            
        return S,T

    def is_absolutely_monotonic(self,r,tol):
        r""" Returns 1 if the TSRK method is absolutely monotonic
            at $z=-r$.

            The method is absolutely monotonic if $(I+rT)^{-1}$ exists
            and
            $$(I+rT)^{-1}T \\ge 0$$
            $$(I+rT)^{-1}S \\ge 0$$

            The inequalities are interpreted componentwise.

            **References**:
                #. [spijker2007]
        """
        S,T = self.spijker_form()
        m=np.shape(T)[0]
        X=np.eye(m)+r*T
        if abs(np.linalg.det(X))<tol: return 0
        P = np.linalg.solve(X,T)
        R = np.linalg.solve(X,S)
        if P.min()<-tol or R.min()<-tol:
            return 0
        else:
            return 1
        # Need an exception here if rhi==rmax



#================================================================
# Functions for analyzing Two-step Runge-Kutta order conditions
#================================================================

def TSRKOrderConditions(p,ind='all'):
    forest=tt.list_trees(p)
    code=[]
    for tree in forest:
        code.append(tsrk_elementary_weight_str(tree)+'-'+str(tree.Emap()))
        code[-1]=code[-1].replace('--','')
        code[-1]=code[-1].replace('1 ','e ')
        code[-1]=code[-1].replace('1)','e)')
    return code

def tsrk_elementary_weight(tree):
    """
        Constructs Butcher's elementary weights 
        for Two-step Runge-Kutta methods
    """
    from sympy import Symbol
    bhat,b,theta=Symbol('bhat',False),Symbol('b',False),Symbol('theta',False)
    ew=bhat*tree.Gprod(tt.Emap,tt.Gprod,betaargs=[TSRKeta,Dmap],alphaargs=[-1])+b*tree.Gprod(TSRKeta,tt.Dmap)+theta*tree.Emap(-1)
    return ew

def tsrk_elementary_weight_str(tree):
    """
        Constructs Butcher's elementary weights 
        for Two-step Runge-Kutta methods
        as numpy-executable strings
    """
    from rooted_trees import Dmap_str
    ewstr='dot(bhat,'+tree.Gprod_str(tt.Emap_str,tt.Gprod_str,betaargs=[TSRKeta_str,Dmap_str],alphaargs=[-1])+')+dot(b,'+tree.Gprod_str(TSRKeta_str,tt.Dmap_str)+')+theta*'+str(tree.Emap(-1))
    ewstr=mysimp(ewstr)
    return ewstr

def TSRKeta(tree):
    from rooted_trees import Dprod
    from sympy import symbols
    raise Exception('This function does not work correctly; use the _str version')
    if tree=='':  return 1
    if tree=='T': return symbols('c',commutative=False)
    return symbols('d',commutative=False)*tree.Emap(-1)+symbols('Ahat',commutative=False)*tree.Gprod(Emap,Dprod,betaargs=[TSRKeta],alphaargs=[-1])+symbols('A',commutative=False)*Dprod(tree,TSRKeta)

def TSRKeta_str(tree):
    """
    Computes eta(t) for Two-step Runge-Kutta methods
    """
    from rooted_trees import Dprod_str, Emap_str
    if tree=='':  return 'e'
    if tree=='T': return 'c'
    return '(d*'+str(tree.Emap(-1))+'+dot(Ahat,'+tree.Gprod_str(Emap_str,Dprod_str,betaargs=[TSRKeta_str],alphaargs=[-1])+')'+'+dot(A,'+Dprod_str(tree,TSRKeta_str)+'))'


#================================================================

def loadTSRK(which='All'):
    r"""
        Load two particular TSRK methods (From [Jackiewicz1995]_).
    """
    TSRK={}
    #================================================
    d=np.array([[-113./88,-103./88]]).T
    theta=-4483./8011
    Ahat=np.array([[1435./352,-479./352],[1917./352,-217./352]])
    A=np.eye(2)
    bhat=np.array([[180991./96132,-17777./32044]]).T
    b=np.array([[-44709./32044,48803./96132]]).T
    TSRK['order4']=TwoStepRungeKuttaMethod(d,theta,A,b,Ahat,bhat)
    #================================================
    d=np.array([-0.210299,-0.0995138])
    theta=-0.186912
    Ahat=np.array([[1.97944,0.0387917],[2.5617,2.3738]])
    A=np.zeros([2,2])
    bhat=np.array([1.45338,0.248242])
    b=np.array([-0.812426,-0.0761097])
    TSRK['order5']=TwoStepRungeKuttaMethod(d,theta,A,b,Ahat,bhat)
    if which=='All': return TSRK
    else: return TSRK[which]


#=====================================================
class TwoStepRungeKuttaError(Exception):
#=====================================================
    """
        Exception class for Two-step Runge Kutta methods.
    """
    def __init__(self,msg='Two-step Runge Kutta Error'):
        self.msg=msg

    def __str__(self):
        return self.msg
#=====================================================

def load_type2_TSRK(s,p,type='Type II'):
    r"""
    Load a TSRK method from its coefficients in an ASCII file
    (usually from MATLAB).  The coefficients are stored in the 
    following order: [s d theta A b Ahat bhat].
    """
    path='/Users/ketch/Research/Projects/MRK/methods/TSRK/'
    file=path+'explicitzr'+str(s)+str(p)+'.tsrk'
    f=open(file,'r')
    coeff=[]
    for line in f:
        for word in line.split():
            coeff.append(float(word))

    s=int(coeff[0])
    A=np.zeros([s,s])
    Ahat=np.zeros([s,s])
    b=np.zeros([s,1])
    bhat=np.zeros([s,1])

    d=np.array(coeff[1:s+1],ndmin=2).T
    theta = coeff[s+1]
    for row in range(s):
        A[row,:] = coeff[s+2+s*row:s+2+s*(row+1)]
    b = np.array(coeff[s**2+s+2:s**2+2*s+2],ndmin=2).T
    for row in range(s):
        Ahat[row,:] = coeff[s**2+2*s+2+s*row:s**2+2*s+2+s*(row+1)]
    bhat = np.array(coeff[2*s**2+2*s+2:2*s**2+3*s+2],ndmin=2).T
    return TwoStepRungeKuttaMethod(d,theta,A,b,Ahat,bhat,type=type)



r"""
Typically, implementation of a Runge-Kutta method requires `s \times N`
memory locations, where `s` is the number of stages of the method and 
`N` is the number of unknowns.  Certain classes of Runge-Kutta methods
can be implemented using substantially less memory, by taking advantage
of special relations among the coefficients.  Three main classes have 
been developed in the literature:

    * 2N (Williamson) methods
    * 2R (van der Houwen/Wray) methods
    * 2S methods

Each of these classes requires only `2\times N` memory locations.
Additional methods have been developed that use more than two 
memory locations per unknown but still provide a substantial savings
over traditional methods.  These are referred to as, e.g., 3S, 3R, 4R,
and so forth.
For a review of low-storage methods, see [ketcheson2010]_ .

In NodePy, low-storage methods are a subclass of explicit Runge-Kutta
methods (and/or explicit Runge-Kutta pairs).  In addition to the usual
properties, they possess arrays of low-storage coefficients.  They 
override the generic RK implementation of time-stepping and use 
special memory-efficient implementations instead.  It should be noted
that, although these low-storage algorithms are implemented, due to
Python language restrictions an extra intermediate copy of the solution
array will be created.  Thus the implementation is not really 
minimum-storage.

At the moment, the following classes are implemented:

    * 2S
    * 2S*
    * 3S*
    * 2S embedded pairs
    * 3S* embedded pairs
    * 2R embedded pairs
    * 3R embedded pairs

**Examples**::

    >>>

"""
from runge_kutta_method import *

#=====================================================
class TwoRRungeKuttaPair(ExplicitRungeKuttaPair):
#=====================================================
    """
        Class for 2R/3R/4R low-storage Runge-Kutta pairs.
        These were developed by van der Houwen, Wray, and Kennedy et. al.

        Only 2R and 3R methods have been implemented so far.
        Also, the implementation is only for embedded pairs
        (not for single methods).

        References:
            * [kennedy2000]_
            * [ketcheson2010]_
    """
    def __init__(self,a,b,bhat,regs=2,
            name='2R Runge-Kutta Method',description=''):
        r"""
            Initializes the 2R method by storing the
            low-storage coefficients and computing the Butcher
            array.

            The coefficients should be specified as follows:

                * For all methods, the weights `b` are used to
                  fill in the appropriate entries in `A`.
                * For 2R methods, *a* is a vector of length `s-1`
                  containing the first subdiagonal of `A`
                * For 3R methods, *a* is a `2\times s-1` array
                  whose first row contains the first subdiagonal of `A`
                  and whose second row contains the second subdiagonal
                  of `A`.
        """
        self.b=b
        self.a=a
        self.bhat=bhat
        m=len(b)
        self.A=np.zeros([m,m])
        for i in range(1,m):
            if regs==2:
                self.A[i,i-1]=a[i-1]
                for j in range(i-1):
                    self.A[i,j]=b[j]
            elif regs==3:
                self.A[i  ,i-1]=a[0,i-1]
                for j in range(i-2):
                    self.A[i,j]=b[j]
                if i<m-1:
                    self.A[i+1,i-1]=a[1,i-1]
            elif regs==4:
                #NEED TO FILL IN
                pass
        self.c=np.sum(self.A,1)
        self.embedded_method=ExplicitRungeKuttaMethod(self.A,self.bhat)
        self.name=name
        self.info=description
        self.lstype=str(regs)+'R+_pair'

    def __step__(self,f,t,u,dt,errest=False,x=None):
        """
            Take a time step on the ODE u'=f(t,u).
            The implementation here is special for 2R/3R low-storage methods
            But it's not really ultra-low-storage yet.

            INPUT:
                f  -- function being integrated
                t  -- array of previous solution times
                u  -- array of previous solution steps
                        (u[i,:] is the solution at time t[i])
                dt -- length of time step to take

            OUTPUT:
                unew -- approximate solution at time t[-1]+dt
        """
        m=len(self); b=self.b; a=self.a
        S2=u[-1]+0.
        S1=u[-1]+0. # by adding zero we get a copy; is there a better way?
        S1=dt*f(t[-1],S1)
        uhat = u[-1]+0.
        if self.lstype.startswith('2'):
            S2=S2+self.b[0]*S1
            uhat = uhat + self.bhat[0]*S1
            for i in range(1,m):
                S1 = S2 + (self.a[i-1]-self.b[i-1])*S1
                S1=dt*f(t[-1]+self.c[i]*dt,S1)
                S2=S2+self.b[i]*S1
                uhat = uhat + self.bhat[i]*S1
            if errest: return S2, np.max(np.abs(S2-uhat))
            else: return S2
        elif self.lstype.startswith('3'):
            S3=S2+self.b[0]*S1
            uhat = uhat + self.bhat[0]*S1
            S1=S3+(self.a[0,0]-self.b[0])*S1
            S2=(S1-S3)/(self.a[0,0]-self.b[0])
            for i in range(1,m-1):
                S1=dt*f(t[-1]+self.c[i]*dt,S1)
                S3=S3+self.b[i]*S1
                uhat = uhat + self.bhat[i]*S1
                S1=S3 + (self.a[0,i]-b[i])*S1 + (self.a[1,i-1]-b[i-1])*S2
                S2=(S1-S3+(self.b[i-1]-self.a[1,i-1])*S2)/(self.a[0,i]-self.b[i])
            S1=dt*f(t[-1]+self.c[m-1]*dt,S1)
            S3=S3+self.b[m-1]*S1
            uhat=uhat+self.bhat[m-1]*S1
            if errest: return S3, np.max(np.abs(S3-uhat))
            else: return S3
        else: print 'Error: only 2R and 3R methods implemented so far!'

#=====================================================
# End of class TwoRRungeKuttaPair
#=====================================================


#=====================================================
class LowStorageRungeKuttaMethod(ExplicitRungeKuttaMethod):
#=====================================================
    """
        Class for low-storage Runge-Kutta methods
        that use Ketcheson's assumption (2S, 2S*, and 3S* methods).

        This class cannot be used for embedded pairs.  Use
        the class LowStorageRungeKuttaPair instead.

        The low-storage coefficient arrays `\beta,\gamma,\delta`
        follow the notation of [ketcheson2010]_ .

        The argument *lstype* must be one of the following values:

            * 2S
            * 2S*
            * 3S*
    """
    def __init__(self,betavec,gamma,delta,lstype,
            name='Low-storage Runge-Kutta Method',description=''):
        r"""
            Initializes the low-storage method by storing the
            low-storage coefficients and computing the Butcher
            coefficients.
       """
        self.betavec=betavec
        self.gamma=gamma
        self.delta=delta
        self.lstype=lstype

        # Two-register methods
        if lstype=='2S' or lstype=='2S*':
            m=len(betavec)-1
            alpha=np.zeros([m+1,m])
            beta =np.zeros([m+1,m])
            for i in range(0,m): beta[i+1,i] = betavec[i+1]
            for i in range(1,m):
                beta[ i+1,i-1] = -beta[i,i-1]*gamma[1][i+1]/gamma[1][i]
                alpha[i+1,i-1] = -gamma[0][i]*gamma[1][i+1]/gamma[1][i]
                alpha[i+1,i  ] = 1. - alpha[i+1,i-1]
            self.A,self.b=shu_osher_to_butcher(alpha,beta)
            self.A=np.tril(self.A,-1)
            self.c=np.sum(self.A,1)

        # Three-register methods
        elif lstype.startswith('3S*'):
            m=len(betavec)-1
            alpha=np.zeros([m+1,m])
            beta =np.vstack([np.zeros(m),np.diag(betavec[1:])])
            for i in range(1,m):
                beta[ i+1,i-1] = -beta[i,i-1]*gamma[1][i+1]/gamma[1][i]
                alpha[i+1,  0] =  gamma[2][i+1]-gamma[2][i]* \
                                    gamma[1][i+1]/gamma[1][i]
                alpha[i+1,i-1] = -gamma[0][i]*gamma[1][i+1]/gamma[1][i]
                alpha[i+1,i  ] = 1. - alpha[i+1,i-1]-alpha[i+1,0]
            self.A,self.b=shu_osher_to_butcher(alpha,beta)
            self.A=np.tril(self.A,-1)
            self.c=np.sum(self.A,1)
        self.name=name
        self.info=description

    def __step__(self,f,t,u,dt):
        """
            Take a time step on the ODE u'=f(t,u).
            The implementation here is special for 2S/2S*/3S* low-storage methods,
            but it's not really ultra-low-storage yet.

            INPUT:
                f  -- function being integrated
                t  -- array of previous solution times
                u  -- array of previous solution steps
                        (u[i,:] is the solution at time t[i])
                dt -- length of time step to take

            OUTPUT:
                unew -- approximate solution at time t[-1]+dt
        """
        m=len(self)
        S1=u[-1]+0. # by adding zero we get a copy; is there a better way?
        S2=np.zeros(np.size(S1))
        if self.lstype.startswith('3S*'): S3=S1+0.
        for i in range(1,m+1):
            S2 = S2 + self.delta[i-1]*S1
            if self.lstype=='2S' or self.lstype=='2S*':
                S1 = self.gamma[0][i]*S1 + self.gamma[1][i]*S2 \
                     + self.betavec[i]*dt*f(t[-1]+self.c[i-1]*dt,S1)
            elif self.lstype.startswith('3S*'):
                S1 = self.gamma[0][i]*S1 + self.gamma[1][i]*S2 \
                     + self.gamma[2][i]*S3 \
                     + self.betavec[i]*dt*f(t[-1]+self.c[i-1]*dt,S1)
        return S1

#=====================================================
# End of class LowStorageRungeKuttaMethod
#=====================================================

#=====================================================
class LowStorageRungeKuttaPair(ExplicitRungeKuttaPair):
#=====================================================
    """
        Class for low-storage embedded Runge-Kutta pairs
        that use Ketcheson's assumption (2S, 2S*, and 3S* methods).

        This class is only for embedded pairs.  Use
        the class LowStorageRungeKuttaMethod for single 2S/3S methods.

        The low-storage coefficient arrays `\beta,\gamma,\delta`
        follow the notation of [ketcheson2010]_ .

        The argument *lstype* must be one of the following values:

            * 2S
            * 2S*
            * 2S_pair
            * 3S*
            * 3S*_pair
 
        The 2S/2S*/3S* classes do not need an extra register for the
        error estimate, while the 2S_pair/3S_pair methods do.
    """
    def __init__(self,betavec,gamma,delta,lstype,bhat=None,
            name='Low-storage Runge-Kutta Pair',description=''):
        """
            Initializes the low-storage pair by storing the
            low-storage coefficients and computing the Butcher
            coefficients.
        """
        self.name=name
        self.info=description
        self.betavec=betavec
        self.gamma=gamma
        self.delta=delta
        self.lstype=lstype

        if lstype=='2S_pair':
            m=len(betavec)-1
            alpha=np.zeros([m+1,m])
            beta =np.zeros([m+1,m])
            for i in range(0,m): beta[i+1,i] = betavec[i+1]
            for i in range(1,m):
                beta[ i+1,i-1] = -beta[i,i-1]*gamma[1][i+1]/gamma[1][i]
                alpha[i+1,i-1] = -gamma[0][i]*gamma[1][i+1]/gamma[1][i]
                alpha[i+1,i  ] = 1. - alpha[i+1,i-1]
            self.A,self.b=shu_osher_to_butcher(alpha,beta)
            self.A=np.tril(self.A,-1)
            self.c=np.sum(self.A,1)
            self.bhat=np.dot(delta,np.vstack([self.A,self.b]))/sum(delta)

        elif lstype=='2S' or lstype=='2S*':
            m=len(betavec)-1
            alpha=np.zeros([m+1,m])
            beta =np.zeros([m+1,m])
            for i in range(0,m): beta[i+1,i] = betavec[i+1]
            for i in range(1,m):
                beta[ i+1,i-1] = -beta[i,i-1]*gamma[1][i+1]/gamma[1][i]
                alpha[i+1,i-1] = -gamma[0][i]*gamma[1][i+1]/gamma[1][i]
                alpha[i+1,i  ] = 1. - alpha[i+1,i-1]
            self.A,self.b=shu_osher_to_butcher(alpha,beta)
            self.A=np.tril(self.A,-1)
            print self.A
            self.c=np.sum(self.A,1)
            self.bhat=bhat

        elif lstype=='3S*_pair':
            m=len(betavec)-1
            alpha=np.zeros([m+1,m])
            beta =np.zeros([m+1,m])
            for i in range(0,m): beta[i+1,i] = betavec[i+1]
            alpha[1,0]=gamma[0][1]+gamma[1][1]+gamma[2][1]
            for i in range(1,m):
                beta[ i+1,i-1]=-beta[i,i-1]*gamma[1][i+1]/gamma[1][i]
                alpha[i+1,  0]= gamma[2][i+1]-gamma[2][i]*gamma[1][i+1]/gamma[1][i]
                alpha[i+1,i-1]=-gamma[0][i]*gamma[1][i+1]/gamma[1][i]
                alpha[i+1,i  ]=1.-alpha[i+1,i-1]-alpha[i+1,0]
            self.A,self.b=shu_osher_to_butcher(alpha,beta)
            self.A=np.tril(self.A,-1)
            self.c=np.sum(self.A,1)
            self.bhat=np.dot(delta[:m+1],np.vstack([self.A,self.b]))/sum(delta)

        elif lstype=='3S*':
            m=len(betavec)-1
            alpha=np.zeros([m+1,m])
            beta =np.vstack([np.zeros(m),np.diag(betavec[1:])])
            for i in range(1,m):
                beta[ i+1,i-1] = -beta[i,i-1]*gamma[1][i+1]/gamma[1][i]
                alpha[i+1,  0] =  gamma[2][i+1]-gamma[2][i]* \
                                    gamma[1][i+1]/gamma[1][i]
                alpha[i+1,i-1] = -gamma[0][i]*gamma[1][i+1]/gamma[1][i]
                alpha[i+1,i  ] = 1. - alpha[i+1,i-1]-alpha[i+1,0]
            self.A,self.b=shu_osher_to_butcher(alpha,beta)
            self.A=np.tril(self.A,-1)
            self.c=np.sum(self.A,1)
            self.bhat=bhat
 
        self.embedded_method=ExplicitRungeKuttaMethod(self.A,self.bhat)

    def __step__(self,f,t,u,dt,errest=False,x=None):
        """
            Take a time step on the ODE u'=f(t,u).

            INPUT:
                f  -- function being integrated
                t  -- array of previous solution times
                u  -- array of previous solution steps
                        (u[i,:] is the solution at time t[i])
                dt -- length of time step to take

            OUTPUT:
                S1 -- approximate solution at time t[-1]+dt
                S2 -- error estimate at time t[-1]+dt

            The implementation here is special for 2S low-storage 
            embedded pairs.
            But it's not really fully low-storage yet.
        """
        m=len(self)
        S1=u[-1]+0. # by adding zero we get a copy; is there a better way?
        S2=np.zeros(np.size(S1))
        if self.lstype.startswith('3S*'): S3=S1+0.; S4=u[-1]+0.
        elif self.lstype=='2S': S3=u[-1]+0.

        for i in range(1,m+1):
            S2 = S2 + self.delta[i-1]*S1
            if self.lstype=='2S_pair':
                S1 = self.gamma[0][i]*S1 + self.gamma[1][i]*S2 \
                     + self.betavec[i]*dt*f(t[-1]+self.c[i-1]*dt,S1)
            elif self.lstype=='2S':
                #Horribly inefficient hack:
                S3 = S3+self.bhat[i-1]*dt*f(t[-1]+self.c[i-1]*dt,S1)
                #End hack
                S1 = self.gamma[0][i]*S1 + self.gamma[1][i]*S2 \
                     + self.betavec[i]*dt*f(t[-1]+self.c[i-1]*dt,S1)
            elif self.lstype=='3S*':
                #Horribly inefficient hack:
                S4 = S4+self.bhat[i-1]*dt*f(t[-1]+self.c[i-1]*dt,S1)
                #End hack
                S1 = self.gamma[0][i]*S1 + self.gamma[1][i]*S2 \
                     + self.gamma[2][i]*S3 \
                     + self.betavec[i]*dt*f(t[-1]+self.c[i-1]*dt,S1)

        #Now put the embedded solution in S2
        if self.lstype=='2S_pair':
            S2=1./sum(self.delta[1:m+1])*(S2+self.delta[m+1]*S1)
        elif self.lstype=='2S': S2=S3
        elif self.lstype=='3S*': S2=S4

        if errest: return S1, np.max(np.abs(S1-S2))
        else: return S1



#=====================================================
#End of LowStorageRungeKuttaPair class
#=====================================================

def load_LSRK(file,lstype='2S',has_emb=False):
    """
        Load low storage methods of the types 2S/2S*/3S*/2S_pair/3S_pair
        from a file containing the low-storage coefficient arrays.
        If has_emb=True, the method has an embedded method that
        requires an extra register.

        The use of both _pair and has_emb seems redundant; this should
        be corrected in the future.
    """
    #Read in coefficients
    f=open(file,'r')
    coeff=[]
    for line in f:
        coeff.append(float(line))
    f.close()
    if has_emb:
        f=open(file+'.bhat','r')
        bhat=[]
        for line in f:
            bhat.append(float(line))

    # Determine number of stages
    if lstype=='2S' or lstype=='2S*': m=int(len(coeff)/3+1) 
    elif lstype=='2S_pair': m=int((len(coeff)+1)/3)
    elif lstype=='3S*': m=int((len(coeff)+6.)/4.)
    elif lstype=='3S*_pair': m=int((len(coeff)+3.)/4.)

    # Fill in low-storage coefficient arrays
    beta=[0.]
    for i in range(m): beta.append(coeff[2*m-3+i])
    gamma=[[0.],[0.,1.]+coeff[0:m-1]]
    if lstype.startswith('3S*'): gamma.append([0,0,0,0]+coeff[3*m-3:4*m-6])
    if lstype=='2S' or lstype=='3S*':  delta=[1.]+coeff[m-1:2*m-3]+[0.]
    elif lstype=='2S*': delta=[1.]+[0.]*len(range(m-1,2*m-3))+[0.]
    elif lstype=='2S_pair': delta=[1.]+coeff[m-1:2*m-3] +[coeff[-2],coeff[-1]]
    elif lstype=='3S*_pair': delta=[1.]+coeff[m-1:2*m-3] +[coeff[-3],coeff[-2],coeff[-1]]
    if lstype=='2S' or lstype=='2S*': 
        for i in range(1,m+1): gamma[0].append(1.-gamma[1][i]*sum(delta[0:i]))
        if has_emb:
            meth = LowStorageRungeKuttaPair(beta,gamma,delta,lstype,bhat=bhat)
        else:
            meth = LowStorageRungeKuttaMethod(beta,gamma,delta,lstype)
    elif lstype=='2S_pair':
        for i in range(1,m+1): gamma[0].append(1.-gamma[1][i]*sum(delta[0:i]))
        meth = LowStorageRungeKuttaPair(beta,gamma,delta,lstype)
    elif lstype.startswith('3S*'):
        for i in range(1,m+1): gamma[0].append(1.-gamma[2][i]
                                        -gamma[1][i]*sum(delta[0:i]))
        if lstype=='3S*':
            if has_emb:
                meth = LowStorageRungeKuttaPair(beta,gamma,delta,lstype,bhat=bhat)
            else:
                meth = LowStorageRungeKuttaMethod(beta,gamma,delta,lstype)

        elif lstype=='3S*_pair':
            meth = LowStorageRungeKuttaPair(beta,gamma,delta,lstype)
    ord=meth.order()
    if lstype=='2S_pair': 
        emb_ord=meth.embedded_order()
        eostr=str(emb_ord)
    else: eostr=''
    m=len(meth)
    lstypename=lstype
    if lstypename=='2S_pair': lstypename='2S'
    meth.name='RK'+str(ord)+'('+eostr+')'+str(m)+'['+lstypename+']'
    return meth

def load_2R(name):
    """
        Loads 2R low-storage methods from the literature.
    """
    if name=='DDAS47':
        fullname='DDAS4()7[2R]'
        descript='2R Method of Tselios \& Simos (2007)'
        b=np.array([0.0941840925477795334,
                    0.149683694803496998,
                    0.285204742060440058,
                    -0.122201846148053668,
                    0.0605151571191401122,
                    0.345986987898399296,
                    0.186627171718797670])
        g=np.array([0.241566650129646868,
                    0.0423866513027719953,
                    0.215602732678803776,
                    0.232328007537583987,
                    0.256223412574146438,
                    0.0978694102142697230])
        a=b[:-1]+g
    elif name=='LDDC46':
        fullname='LDDC4()6[2R]'
        b=np.array([0.10893125722541,
                    0.13201701492152,
                    0.38911623225517,
                    -0.59203884581148,
                    0.47385028714844,
                    0.48812405426094])
        g=np.array([0.17985400977138,
                    0.14081893152111,
                    0.08255631629428,
                    0.65804425034331,
                    0.31862993413251])
        descript='2R Method of Calvo'
        a=b[:-1]+g
    elif name=='RK45[2R]C':
        fullname='RK4(3)5[2R+]C'
        descript='A 2R Method of Kennedy et. al.'
        regs=2
        a=np.array([970286171893./4311952581923,
                    6584761158862./12103376702013,
                    2251764453980./15575788980749,
                    26877169314380./34165994151039])
        b=np.array([1153189308089./22510343858157,
                    1772645290293./4653164025191,
                    -1672844663538./4480602732383,
                    2114624349019./3568978502595,
                    5198255086312./14908931495163])
        bhat=np.array([1016888040809./7410784769900,
                       11231460423587./58533540763752,
                      -1563879915014./6823010717585,
                       606302364029./971179775848,
                       1097981568119./3980877426909])
    elif name=='RK58[3R]C':
        fullname='RK5(4)8[3R+]C'
        descript='A 3R Method of Kennedy et. al.'
        regs=3
        a=np.array([[141236061735./3636543850841,
                    7367658691349./25881828075080,
                    6185269491390./13597512850793,
                    2669739616339./18583622645114,
                    42158992267337./9664249073111,
                    970532350048./4459675494195,
                    1415616989537./7108576874996], #1st subdiagonal
                  [-343061178215./2523150225462,
                     -4057757969325./18246604264081,
                      1415180642415./13311741862438,
                     -93461894168145./25333855312294,
                      7285104933991./14106269434317,
                     -4825949463597./16828400578907,0.]]) #2nd subdiagonal
        b=np.array([514862045033./4637360145389,
                    0.,
                    0.,
                    0.,
                    2561084526938./7959061818733,
                    4857652849./7350455163355,
                    1059943012790./2822036905401,
                    2987336121747./15645656703944])
        bhat=np.array([1269299456316./16631323494719,
                       0.,
                       2153976949307./22364028786708,
                       2303038467735./18680122447354,
                       7354111305649./15643939971922,
                       768474111281./10081205039574,
                       3439095334143./10786306938509,
                       -3808726110015./23644487528593])
    elif name=='RK59[2R]C':
        fullname='RK5(4)9[2R+]C'
        descript='A 2R Method of Kennedy et. al.'
        regs=2
        a=np.array([1107026461565./5417078080134,
         38141181049399./41724347789894,
         493273079041./11940823631197,
         1851571280403./6147804934346,
         11782306865191./62590030070788,
         9452544825720./13648368537481,
         4435885630781./26285702406235,
         2357909744247./11371140753790])
        b=np.array([ 2274579626619./23610510767302,
                     693987741272./12394497460941,
                   - 347131529483./15096185902911,
                     1144057200723./32081666971178,
                     1562491064753./11797114684756,
                     13113619727965./44346030145118,
                     393957816125./7825732611452,
                     720647959663./6565743875477,
                     3559252274877./14424734981077])
        bhat=np.array([ 266888888871./3040372307578,
                         34125631160./2973680843661,
                       - 653811289250./9267220972999,
                         323544662297./2461529853637,
                         1105885670474./4964345317203,
                         1408484642121./8758221613943,
                         1454774750537./11112645198328,
                         772137014323./4386814405182,
                         277420604269./1857595682219])
    return TwoRRungeKuttaPair(a,b,bhat,regs,fullname,description=descript)



"""
Class for three-step Runge-Kutta methods, and various functions related to them.
This still contains errors, particularly in the order conditions!

AUTHOR: David Ketcheson (06-21-2009)

EXAMPLES:

REFERENCES:
    [hairer1993]

.. warning::

    The order condition functions here have not been carefully tested
    and may not generate correct order conditions!
"""
from __future__ import division
from general_linear_method import GeneralLinearMethod
import numpy as np
import rooted_trees as tt
from sympy import Symbol
from strmanip import *

#=====================================================
class ThreeStepRungeKuttaMethod(GeneralLinearMethod):
#=====================================================
    """ General class for Three-step Runge-Kutta Methods """
    def __init__(self,D,theta,A,b):
        r"""
            Initialize a 3-step Runge-Kutta method.  The representation
            uses the form and notation of [Ketcheson2009]_.
        """
        self.D,self.theta,self.A,self.b=D,theta,A,b

    def order(self,tol=1.e-13):
        r""" 
            Return the order of a Three-step Runge-Kutta method.
            Computed by computing the elementary weights corresponding
            to the appropriate rooted trees.
        """
        p=0
        while True:
            z=self.orderConditions(p+1)
            if np.any(abs(z)>tol): return p
            p=p+1

    def orderConditions(self,p):
        r"""
            Evaluate the order conditions corresponding to rooted trees
            of order $p$.

            **Output**:
                - A vector $z$ of residuals (the amount by which each
                  order condition is violated)
        """
        from numpy import dot
        D,theta,A,b=self.D,self.theta,self.A,self.b
        l=np.array(range(0,3))
        e=np.ones(A.shape[1])
        c=dot(A,e)-dot(D,l)
        code=ThSRKOrderConditions(p)
        z=np.zeros(len(code))
        for i in range(len(code)):
            exec('z[i]='+code[i])
            #print p,z
        return z

#================================================================
# Functions for analyzing Three-step Runge-Kutta order conditions
#================================================================

def ThSRKOrderConditions(p,ind='all'):
    forest=tt.list_trees(p)
    code=[]
    for tree in forest:
        code.append(thsrk_elementary_weight_str_matlab(tree)+'-'+str(tree.Emap()))
        code[-1]=code[-1].replace('--','')
        code[-1]=code[-1].replace('1 ','e ')
        code[-1]=code[-1].replace('1)','e)')
    return code

def thsrk_elementary_weight(tree):
    """
        Constructs Butcher's elementary weights 
        for Three-step Runge-Kutta methods.
    """
    b,theta=Symbol('b',False),Symbol('theta',False)
    ew=b*tree.Gprod(tt.ThSRKeta,tt.Dmap)+theta2*tree.Emap(-1)+theta3*tree.Emap(-2)
    return ew

def thsrk_elementary_weight_str(tree):
    """
        Constructs Butcher's elementary weights 
        for Two-step Runge-Kutta methods
        as numpy-executable strings
    """
    ewstr='dot(b,'+tree.Gprod_str(ThSRKeta_str,tt.Dmap_str)+')+('+str(tree.Emap(-1))+')*theta2+('+str(tree.Emap(-2))+')*theta3'
    ewstr=mysimp(ewstr)
    return ewstr

def thsrk_elementary_weight_str_matlab(tree):
    """
        Constructs Butcher's elementary weights 
        for Two-step Runge-Kutta methods
        as matlab-executable strings
    """
    ewstr="b'*("+tree.Gprod_str(tt.ThSRKeta_str_matlab,tt.Dmap_str)+')+('+str(tree.Emap(-1))+')*theta2+('+str(tree.Emap(-2))+')*theta3'
    ewstr=mysimp(ewstr)
    ewstr=ewstr.replace('**','.^')
    return ewstr

def ThSRKeta(tree):
    from rooted_trees import Dprod
    from sympy import symbols
    raise Exception('This function does not work correctly; use the _str version')
    if tree=='':  return 1
    if tree=='T': return symbols('c',commutative=False)
    return symbols('d2',commutative=False)*tree.Emap(-1)+symbols('d3',commutative=False)*tree.Emap(-2)+symbols('A',commutative=False)*Dprod(tree,ThSRKeta)

def ThSRKeta_str(tree):
    """
    Computes eta(t) for Two-step Runge-Kutta methods -- Python string
    """
    from rooted_trees import Dprod_str
    if tree=='':  return 'e'
    if tree=='T': return 'c'
    return '(d2*'+str(tree.Emap(-1))+'+(d3*'+str(tree.Emap(-2))+'+dot(A,'+Dprod_str(tree,ThSRKeta_str)+'))'

def ThSRKeta_str_matlab(tree):
    """
    Computes eta(t) for Two-step Runge-Kutta methods -- Matlab string
    """
    from rooted_trees import Dprod_str
    if tree=='':  return 'e'
    if tree=='T': return 'c'
    return "("+str(tree.Emap(-1))+")*d2+("+str(tree.Emap(-2))+")*d3+(A*"+Dprod_str(tree,ThSRKeta_str_matlab)+')'


#================================================================



"""
Class for Downwind Runge-Kutta methods, and various functions related to them.

AUTHOR: David Ketcheson (03-10-2010)

EXAMPLES:

REFERENCES:
    [higueras2005]_
"""
from __future__ import division
from general_linear_method import GeneralLinearMethod
import runge_kutta_method as rk
import utils
import numpy as np

#=====================================================
class DownwindRungeKuttaMethod(GeneralLinearMethod):
#=====================================================
    """ General class for Downwind Runge-Kutta Methods """
    def __init__(self,A=None,At=None,b=None,bt=None,alpha=None,beta=None,
            alphat=None,betat=None,
            name='downwind Runge-Kutta Method',description=''):
        r"""
            Initialize a downwind Runge-Kutta method.  The representation
            uses the form and notation of [Ketcheson2010]_.

            \\begin{align*}
            y^n_i = & u^{n-1} + \\Delta t \\sum_{j=1}^{s}
            (a_{ij} f(y_j^{n-1}) + \\tilde{a}_{ij} \\tilde{f}(y_j^n)) & (1\\le j \\le s) \\\\
            \\end{align*}

        """
        butchform=[x is not None for x in [A,At,b,bt]]
        SOform=[x is not None for x in [alpha,alphat,beta,betat]]
        if not ( ( all(butchform) and not (True in SOform) ) or
                    ( (not (True in butchform)) and all(SOform) ) ):
            raise rk.RungeKuttaError("""To initialize a Runge-Kutta method,
                you must provide either Butcher arrays or Shu-Osher arrays,
                but not both.""")
        if A is not None: #Initialize with Butcher arrays
            # Check that number of stages is consistent
            m=np.size(A,0) # Number of stages
            if m>1:
                if not np.all([np.size(A,1),np.size(b)]==[m,m]):
                   raise rk.RungeKuttaError(
                    'Inconsistent dimensions of Butcher arrays')
            else:
                if not np.size(b)==1:
                    raise rk.RungeKuttaError(
                     'Inconsistent dimensions of Butcher arrays')
        elif alpha is not None: #Initialize with Shu-Osher arrays
            A,At,b,bt=downwind_shu_osher_to_butcher(alpha,alphat,beta,betat)
        # Set Butcher arrays
        if len(np.shape(A))==2: 
          self.A=A
          self.At=At
        else: 
          self.A =np.array([A ]) #Fix for 1-stage methods
          self.At=np.array([At])
        self.b=b;
        self.bt=bt
        self.c=np.sum(self.A,1)-np.sum(self.At,1)
        self.name=name
        self.info=description
        self.underlying_method=rk.RungeKuttaMethod(self.A-self.At,self.b-self.bt)

    def __repr__(self): 
        """
        Pretty-prints the Butcher array in the form:
          |   |
        c | A | At
        ___________
          | b | bt
        """
        s=self.name+'\n'+self.info+'\n'
        for i in range(len(self)):
            s+='%6.3f |' % self.c[i]
            for j in range(len(self)):
                s+=' %6.3f' % self.A[i,j]
            s+=' | '    
            for jj in range(len(self)):
                s+=' %6.3f' % self.At[i,jj]    
            s+='\n'
        s+='_______|'+('_______'*2*len(self))+('_____')+'\n'
        s+= '       |'
        for j in range(len(self)):
            s+=' %6.3f' % self.b[j]
        s+=' | '
        for jj in range(len(self)):
            s+=' %6.3f' % self.bt[jj]
        return s
 
    def __len__(self):
        """
            The length of the method is the number of stages.
        """
        return np.size(self.A,0) 

    def order(self,tol=1.e-13):
        r""" 
            Return the order of a Downwind Runge-Kutta method.
        """
        return self.underlying_method.order(tol)

    def absolute_monotonicity_radius(self,acc=1.e-10,rmax=200,
                    tol=3.e-16):
        r""" 
            Returns the radius of absolute monotonicity
            of a Runge-Kutta method.
        """
        from utils import bisect
        
        r=bisect(0,rmax,acc,tol,self.is_absolutely_monotonic)
        return r

    def is_absolutely_monotonic(self,r,tol):
        r""" Returns 1 if the downwind Runge-Kutta method is 
            absolutely monotonic at $z=-r$.

            The method is absolutely monotonic if $(I+rK+rKt)^{-1}$ exists
            and
            $$(I+rK+rKt)^{-1}K \\ge 0$$
            $$(I+rK+rKt)^{-1}Kt \\ge 0$$
            $$(I+rK+rKt)^{-1} e_m \\ge 0$$

            where $e_m$ is the m-by-1 vector of ones and
                  K=[ K  0
                     b^T 0].

            The inequalities are interpreted componentwise.

        """
        m=len(self)
        K  =np.hstack([np.vstack([self.A  ,self.b ]),np.zeros([m+1,1])])
        Kt =np.hstack([np.vstack([self.At ,self.bt]),np.zeros([m+1,1])])
        X=np.eye(len(self)+1) + r*(K+Kt)
        beta =r*np.linalg.solve(X, K)
        betat=r*np.linalg.solve(X,Kt)
        ech=np.linalg.solve(X,np.ones(m+1))
        if min(beta.min(),betat.min(),ech.min())<-tol:
            return 0
        else:
            return 1
        # Need an exception here if rhi==rmax

#================================================================

def loadDWRK(which='All'):
    r"""
        Load some particular DWRK method.
    """
    DWRK={}
    if which=='All': return TSRK
    else: return TSRK[which]

def opt_dwrk(r):
    #a11=(r**2-2*r-2)/(2.*r)
    at12=(r**2-4*r+2)/(2.*r)
    #a21=(r-2.)/2.
    alpha=np.array([[2./r,0],[1.,0],[0,1]])
    alphat=np.array([[0.,(r**2-4*r+2)/(r**2-2*r)],[0,0],[0,0]])
    beta=alpha/r
    betat=alphat/r
    return DownwindRungeKuttaMethod(alpha=alpha,beta=beta,alphat=alphat,betat=betat)


def downwind_shu_osher_to_butcher(alpha,alphat,beta,betat):
    r""" Accepts a Shu-Osher representation of a downwind Runge-Kutta
        method and returns the Butcher coefficients 

        \\begin{align*}
        A  = & (I-\\alpha_0-\\alphat_0)^{-1} \\beta_0 \\\\
        At = & (I-\\alpha_0-\\alphat_0)^{-1} \\betat_0 \\\\
        b = & \\beta_1 + (\\alpha_1 + \\alphat_1) * A
        \\end{align*}

        **References**:  
             #. [gottlieb2009]_
    """
    m=np.size(alpha,1)
    if not np.all([np.size(alpha,0),np.size(beta,0),
                    np.size(beta,1)]==[m+1,m+1,m]):
        raise RungeKuttaError(
             'Inconsistent dimensions of Shu-Osher arrays')
    X=np.eye(m)-alpha[0:m,:]-alphat[0:m,:]
    A=np.linalg.solve(X,beta[0:m,:])
    At=np.linalg.solve(X,betat[0:m,:])
    b=beta[m,:]+np.dot(alpha[m,:]+alphat[m,:],A)
    bt=betat[m,:]+np.dot(alpha[m,:]+alphat[m,:],At)
    return A,At,b,bt





#=====================================================
class ODESolver:
#=====================================================
    """ Top-level class for numerical ODE solvers """
    def __init__(self):
        pass

    def __step__(self):
        raise NotImplementedError

    def __call__(self,ivp,t0=0,N=5000,dt=None,errtol=None,controllertype='P',x=None,diagnostics=False):
        """
            Calling an ODESolver numerically integrates the ODE
            u'(t) = f(t,u(t)) with initial value u(0)=u0 from time
            t0 up to time T using the solver.

            The timestep can be controlled in any of three ways:

                1. By specifying N, the total number of steps to take.
                   Then dt = (T-t0)/N.
                2. By specifying dt directly.
                3. For methods with an error estimate (e.g., RK pairs),
                   by specifying an error tolerance.  Then the step
                   size is adjusted using a PI-controller to achieve
                   the requested tolerance.  In this case, dt should
                   also be specified and determines the value of the
                   initial timestep.

            The argument x is used to pass any additional arguments required
            for the RHS function f.

            **Input**:
                - ivp -- An IVP instance (the initial value problem to be solved)
                - t0 -- The initial time from which to integrate
                - N -- The # of steps to take (using a fixed step size)
                - dt -- The step size to use
                - errtol -- The local error tolerance to be observed (using
                  adaptive stepping).  This requires that the method have an
                  error estimator, such as an embedded Runge-Kutta method.
                - controllerType -- The type of adaptive step size control
                  to be used; available options are 'P' and 'PI'.  See
                  [hairer1993b]_ for details.
                - diagnostics -- if True, return the number of rejected steps
                  and a list of step sizes used, in addition to the solution
                  values and times.

            **Output**:
                - t -- A list of solution times
                - u -- A list of solution values

            TODO: 

                * Implement an option to not keep all output (for efficiency).
                * Option to keep error estimate history
        """
        numself = self.__num__()
        f=ivp.rhs; u0=ivp.u0; T=ivp.T
        u=[u0]; t=[t0]; dthist=[]
        rejected_steps=0

        if errtol is None:      # Fixed-timestep mode
            if dt is None: dt=(T-t0)/N
            while t[-1]<T:
                if t[-1]+dt>T: dt=T-t[-1]
                if x is not None: u.append(numself.__step__(f,t,u,dt,x=x))
                else: u.append(numself.__step__(f,t,u,dt))
                t.append(t[-1]+dt)

        else:                   # Error-control mode
            p=numself.embedded_method.order()
            alpha = 0.7/p; beta = 0.4/p; kappa = 0.9
            facmin = 0.2; facmax = 5.
            errestold = errtol
            errest=1.

            if dt is None: print 'ERROR: Must specify initial timestep for error-control mode'
            while t[-1]<T:
                if t[-1]+dt>T: dt=T-t[-1]
                unew,errest = numself.__step__(f,t,u,dt,errest=True,x=x)
                if errest<=errtol:      # Step accepted
                    u.append(unew)
                    t.append(t[-1]+dt)
                    errestold = errest  #Should this happen if step is rejected?
                    dthist.append(dt)
                else: rejected_steps+=1

                if controllertype=='P':
                  #Compute new dt using P-controller
                  facopt = (errtol/errest)**alpha 

                elif controllertype=='PI':
                  #Compute new dt using PI-controller
                  facopt = ((errtol/errest)**alpha
                            *(errestold/errtol)**beta)
                else: print 'Unrecognized time step controller type'

                dt = dt * min(facmax,max(facmin,kappa*facopt))

        if diagnostics==False: return t,u
        else: return t,u,rejected_steps,dthist



"""
Functions for running convergence and performance tests.
"""
import matplotlib.pyplot as pl
import numpy as np
from nodepy import runge_kutta_method as rk

def ctest(methods,ivp,grids=[20,40,80,160,320,640]):
    """
        Runs a convergence test, integrating a single initial value problem
        using a sequence of fixed step sizes and a set of methods.
        Creates a plot of the resulting errors versus step size for each method.

        **Inputs**:
            - methods -- a list of ODEsolver instances
            - ivp     -- an IVP instance
            - grids   -- a list of grid sizes for integration.
                      optional; defaults to [20,40,80,160,320,640]

        **Example**::

            import runge_kutta_method as rk
            from ivp import *
            rk44=rk.loadRKM('RK44')
            myivp=load_ivp('nlsin')
            err=ctest(rk44,myivp)

        TODO: 
            - Option to plot versus f-evals or dt

    """
    pl.clf(); pl.hold(True)
    # In case just one method is passed in (and not as a list):
    if not isinstance(methods,list): methods=[methods]
    err=[]
    try:
        exsol = ivp.exact(ivp.T)
    except:
        bs5=rk.loadRKM('BS5')
        bigN=grids[-1]*4
        print 'solving on fine grid with '+str(bigN)+' points'
        t,u=bs5(ivp,N=bigN)
        print 'done'
        exsol = u[-1].copy()
    for method in methods:
        print "solving with %s" % method.name
        err0=[]
        for i,N in enumerate(grids):
            t,u=method(ivp,N=N)
            err0.append(np.linalg.norm(u[-1]-exsol))
        err.append(err0)
        work=[grid*len(method) for grid in grids]
        pl.loglog(work,err0,label=method.name,linewidth=3)
    pl.xlabel('Function evaluations')
    pl.ylabel('Error at $t_{final}$')
    pl.legend(loc='best')
    pl.hold(False)
    pl.draw()
    return err


def ptest(methods,ivps,tols=[1.e-1,1.e-2,1.e-4,1.e-6]):
    """
        Runs a performance test, integrating a set of problems with a set
        of methods using a sequence of error tolerances.  Creates a plot 
        of the error achieved versus the amount of work done (number of
        function evaluations) for each method.

        **Input**:
            * methods -- a list of ODEsolver instances
                      Note that all methods must have error estimators.
            * ivps    -- a list of IVP instances
            * tols    -- a specified list of error tolerances (optional)

        **Example**::

            import runge_kutta_method as rk
            from ivp import *
            bs5=rk.loadRKM('BS5')
            myivp=load_ivp('nlsin')
            work,err=ptest(bs5,myivp)

    """
    pl.clf(); pl.draw(); pl.hold(True)
    # In case just one method is passed in (and not as a list):
    if not isinstance(methods,list): methods=[methods]
    if not isinstance(ivps,list): ivps=[ivps]
    err=np.ones([len(methods),len(tols)])
    work=np.zeros([len(methods),len(tols)])
    for ivp in ivps:
        print "solving problem %s" % ivp
        try:
            exsol = ivp.exact(ivp.T)
        except:
            bs5=rk.loadRKM('BS5')   #Use Bogacki-Shampine RK for fine solution
            lowtol=min(tols)/100.
            print 'solving for "exact" solution with tol= '+str(lowtol)
            t,u=bs5(ivp,errtol=lowtol,dt=ivp.dt0)
            print 'done'
            exsol = u[-1].copy()
        for imeth,method in enumerate(methods):
            print 'Solving with method '+method.name
            workperstep = len(method)-method.is_FSAL()
            for jtol,tol in enumerate(tols):
                t,u,rej,dt=method(ivp,errtol=tol,dt=ivp.dt0,diagnostics=True,controllertype='P')
                print str(rej)+' rejected steps'
                err[imeth,jtol]*= np.max(np.abs(u[-1]-exsol))
                #FSAL methods save on accepted steps, but not on rejected:
                work[imeth,jtol]+= len(t)*workperstep+rej*len(method)
    for imeth,method in enumerate(methods):
        for jtol,tol in enumerate(tols):
            err[imeth,jtol]=err[imeth,jtol]**(1./len(ivps))
    for imeth,method in enumerate(methods):
        pl.semilogy(work[imeth,:],err[imeth,:],label=method.name,linewidth=3)
    pl.xlabel('Function evaluations')
    pl.ylabel('Error at $t_{final}$')
    pl.legend(loc='best')
    pl.hold(False)
    pl.draw()
    return work,err
